SSEModel class

Model for representing an SSE event.

Constructors

SSEModel({String? data, String? id, String? event})
Constructor for SSEModel.
SSEModel.fromData(String data)
Constructs an SSEModel from a data string.

Properties

data String?
Event data.
getter/setter pair
event String?
Event name.
getter/setter pair
hashCode int
The hash code for this object.
no setterinherited
id String?
ID of the event.
getter/setter pair
runtimeType Type
A representation of the runtime type of the object.
no setterinherited

Methods

noSuchMethod(Invocation invocation) → dynamic
Invoked when a nonexistent method or property is accessed.
inherited
toString() String
A string representation of this object.
inherited

Operators

operator ==(Object other) bool
The equality operator.
inherited