RetryOptions class
Constructors
- RetryOptions({int maxRetryTime = 5000, int minRetryTime = 5000, int maxRetry = 5, Future limitReachedCallback()?})
Properties
- hashCode → int
-
The hash code for this object.
no setterinherited
- limitReachedCallback ↔ Future Function()?
-
getter/setter pair
- maxRetry ↔ int
-
getter/setter pair
- maxRetryTime ↔ int
-
getter/setter pair
- minRetryTime ↔ int
-
getter/setter pair
- runtimeType → Type
-
A representation of the runtime type of the object.
no setterinherited
Methods
-
noSuchMethod(
Invocation invocation) → dynamic -
Invoked when a nonexistent method or property is accessed.
inherited
-
toString(
) → String -
A string representation of this object.
inherited
Operators
-
operator ==(
Object other) → bool -
The equality operator.
inherited